#! /usr/bin/env python
'''
Module Name:    AtmelTouchFwFlash.py

Description:    Class for Flashing Atmel Touch FW and XCFG Updates

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2015 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Created :  2 December 013

'''
import os



from monitorupgrade import TLog as TLOG
from monitorupgrade import Utils as utils
from monitorupgrade import Configs as Configs

###import subprocess
import time

EXIT_SUCCESS = 0

#
# Dawu Utilities    
#
MXT_APP = '/usr/bin/mxt-app'
#MXT_APP = '/opt/toshiba/mus/op_utils/mxtapp/mxt-app'
CONFIG_UTIL = '/opt/toshiba/mus/op_utils/configutil/configutil'
ATMEL_FW_RECOVERY_FILE = utils.ATMEL_FW_RECOVERY_FILE
ATMEL_XCFG_RECOVERY_FILE = utils.ATMEL_XCFG_RECOVERY_FILE
JIGSAW_APP = '/opt/toshiba/mus/op_utils/hlprs/extract'

# Set this to true to enable python debugger 'pudb'
PUDBG=False

# Logging
MXT_APP_LOG_LEVEL = 2
MUS_LOG_FILE_DIR = utils.MUS_LOG_FILE_DIR

# Directory internal to UP HWID directory
FWDIR_NAME = 'fw'

#
# FW and XCFG Recovery File lines
#
UP_PATH="UP_PATH"
FW_PATH="FW_PATH"
XCFG_PATH="XCFG_PATH"
CHIP_SERIAL="CHIP_SERIAL"

#
# Code Flags
#

# Verbose printed debug
ADBG=True

if PUDBG:
    try:
        import pudb
    except ImportError:
        pass

# Reboot if we discovered the chip earlier but now it is not present. Default is just abandon the task 
CHIP_SANITY_MEANS_REBOOT=False



#==========================================================================================================
# AtmelTouchFwFlashTask - Class for Flashing Atmel Touch FW Update
#==========================================================================================================        
class AtmelTouchFwFlashTask:
    up_path=''
    fw_name_dot_up=''
    controller=''
    serial=''
    hwid=''
    
    def __init__(self,path,name,controller,serial,hwid):
        print 'AtmelFwFlashTask Constructed'
        TLOG.logger.info('AtmelTouchFwFlashTask Constructed')
        # This is the manifest's version of the up path - has quotes
        self.up_path = path.strip('\'')
        # This is the manifest's version of the up name - has quotes
        self.fw_name_dot_up = name.strip('\'')
        self.controller = controller
        self.serial = serial
        self.hwid = hwid
    
    def extractFirmware(self, up_fw_path,fw_name_dot_enc):       
        TLOG.logger.info('AtmelTouchFwFlash::extractFirmware() -----------------START---------------')
        dec_cmd = JIGSAW_APP + ' ' + up_fw_path + ' ' + fw_name_dot_enc + ' > /dev/null'                      
        #dec_cmd = JIGSAW_APP + ' ' + up_fw_path + ' ' + fw_name_dot_enc           
        if ADBG: print dec_cmd
        ret = utils.runShell(dec_cmd)
        TLOG.logger.info('AtmelTouchFwFlash::extractFirmware()  - ret:  ' + str (ret))
        return ret
        
    def deleteFirmware(self, fw_path):
        TLOG.logger.info('AtmelTouchFwFlash::deleteFirmware() -----------------START---------------')
        
        cmd = 'rm '   + fw_path
        
        ret = utils.runShell(cmd)   
        TLOG.logger.info('AtmelTouchFwFlash::deleteFirmware() - ret:  ' + str (ret) )
        
    def run(self):
        
        TLOG.logger.info('AtmelTouchFwFlash::run() -----------------START---------------')

        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        
        if ADBG: print 'AtmelTouchFwFlashTask::run() ', self.up_path, self.fw_name_dot_up, self.controller
        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        

        try:
            if PUDBG: pudb.set_trace()
        except:
            pass
        
        usb_devid_tmp = 'usb:'  + self.controller

        up_path = self.up_path
        up_fw_path =  os.path.join(self.up_path,FWDIR_NAME)
        
         
        tskcfg = Configs.UpdateTaskConfig(up_path)
        #
        # Manifest's .ENC firmare file
        #
        fw_name_dot_enc = tskcfg.getAtmelFirmwareName()
        fw_name_dot_enc = fw_name_dot_enc.strip('\'')
        fw_path = (os.path.join(up_fw_path , fw_name_dot_enc))

        #
        # Doubly encrypted .UP firmware file
        #
        fw_name_dot_up = self.fw_name_dot_up.strip('\'')
               
        if ADBG: print 'usb_devid=',usb_devid_tmp
        if ADBG: print 'up_path=',up_path
        if ADBG: print 'up_fw_path=',up_fw_path
        if ADBG: print 'fw_name_dot_enc=',fw_name_dot_enc
        if ADBG: print 'fw_path=',fw_path
        if ADBG: print 'fw_name_dot_up=',fw_name_dot_up

        #
        # Check for mxt-app always .. absence is an unrecoverable error
        #
        if not utils.doesFileExist(MXT_APP):
            print '[AtmelTouchFwFlashTask] - Bad OP - No MXT_APP found!'
            TLOG.logger.error('AtmelTouchFwFlashTask::run() - Bad OP - No MXT_APP found!')
            raise RuntimeError('Bad OP - No MXT_APP utility found!')
        

        ###################
        # PREPARE FLASH #
        ###################
        #
        # Firmware file discovery and decryption
        #
        if not utils.doesFileExist(JIGSAW_APP):
            print '[AtmelTouchFwFlashTask] - Bad OP - No JIGSAW_APP found!'
            TLOG.logger.error('AtmelTouchFwFlashTask::run() - Bad OP - No JIGSAW_APP found!')
            raise RuntimeError('Bad OP - No JIGSAW_APP utility script found!')
        
        de_fw_path = os.path.join(up_fw_path, fw_name_dot_up)
        
        print 'Looking for doubly encrypted firmware file :', de_fw_path
        TLOG.logger.info('AtmelTouchFwFlashTask::run() Looking for doubly encrypted firmware file : '+de_fw_path)

        if not utils.doesFileExist(de_fw_path):
            print '[AtmelTouchFwFlash] - Bad UP - No doubly encrypted firmware file found!'
            TLOG.logger.error('AtmelTouchFwFlash::run() - Bad UP - No doubly encrypted firmware file found!')
            raise RuntimeWarning('Bad UP - No doubly encrypted firmware file found!')
        
        print 'Found doubly encrypted firmware file : ', fw_name_dot_up, ' proceeding to decrypt'
        TLOG.logger.info('AtmelTouchFwFlash::run() Found doubly encrypted firmware file : ' + fw_name_dot_up+' proceeding to decrypt')

        ret = self.extractFirmware(up_fw_path, fw_name_dot_up)
        
        if ret != 0:
            print '[AtmelTouchFwFlash] - Double decrypt of Atmel .UP firmware file failed with error ', ret
            TLOG.logger.error('AtmelTouchFwFlash::run() - Double decrypt of Atmel .UP firmware file failed with error '+str(ret))
            raise RuntimeWarning('Double decrypt of Atmel .UP firmware file failed!')
 

        ##########################
        # RECOVERY FILE CREATE #
        ##########################
        
        print 'Creating FW recovery file : ', ATMEL_FW_RECOVERY_FILE
        TLOG.logger.info('AtmelTouchFwFlash::run() - Creating recovery file : ' + ATMEL_FW_RECOVERY_FILE)

        try:
            up_path_line = UP_PATH + "=" + up_path + "\n"
            fw_path_line = FW_PATH + "=" + fw_path + "\n"
            chip_serial_line = CHIP_SERIAL + "=" + self.serial + "\n"
            #
            f = open(ATMEL_FW_RECOVERY_FILE,'w')
            f.write(up_path_line)
            f.write(fw_path_line)
            f.write(chip_serial_line)
            f.close()
            
        except IOError:
            print 'Creating FW recovery file failed'
            TLOG.logger.info('AtmelTouchFwFlash::run() - Creating FW recovery file failed')
            # Try remove to be certain
            os.remove(ATMEL_FW_RECOVERY_FILE)
            raise RuntimeWarning('Creating recovery file failed')
        #         

        
        ######################
        # SANITY CHECK CHIP #
        ######################      
        #
        # We had an Atmel chip earlier when we assigned tasks - check we still have it
        #
        tmp_controller = self.controller
        self.controller = utils.getUsbDevidForSerial(self.serial)
                    
        if (self.controller=='UNKNOWN'):
            print 'Chip not present any more - Devid: ', tmp_controller,' Serial :',self.serial
            TLOG.logger.warn('AtmelTouchFwFlash::run() - Chip not present any more - '\
                             +' Old Devid: '+tmp_controller\
                             +' Serial: '+self.serial)
            if CHIP_SANITY_MEANS_REBOOT:
                # Exit and reboot
                return utils.ERROR_CODE_ATMEL_DEVICE_NOT_EXIST
            else:
                # Remove the recovery file and abandon task
                os.remove(ATMEL_FW_RECOVERY_FILE)
                raise RuntimeWarning('Chip not present any more')
            #
        else:
            print 'Chip still present - Old Devid: ', tmp_controller,' Devid: ',self.controller,' Serial: ',self.serial
            TLOG.logger.info('AtmelTouchFwFlash::run() - Chip still present - '\
                             +' Old Devid: '+tmp_controller\
                             +' Devid: '+self.controller\
                             +' Serial: '+self.serial)
            
        #
        # Set up usb devid in mxt-app format: 'usb:hhh-ddd'
        #
        usb_devid_flash = 'usb:'  + self.controller
        #
        
                
         
        #####################
        # FIRMWARE UPDATE #
        #####################
 
        #
        # DO FLASH
        #
        print "[AtmelTouchFwFlash] - Starting Firmware update now..."
        print "[AtmelTouchFwFlash] - This will take some time. Please wait ..."
	TLOG.logger.info('AtmelTouchFwFlash::run() -----------------ENUMERATION------------\n')
	utils.find614X()        
 	# Need delay, if not the mxt-app will encounter errno 19 sometimes
        time.sleep(20)
                 
        print "[AtmelTouchFwFlash] - Flashing firmware to device:" + usb_devid_flash
        mxt_app_cmd = MXT_APP + ' -d ' + usb_devid_flash + ' --flash ' + fw_path + ' --verbose=' + str(MXT_APP_LOG_LEVEL)
        print mxt_app_cmd               

        ret = utils.runSubprocess(mxt_app_cmd,'MXT-APP')

        #
        # Bomb out and do a recovery
        #
        if ret != EXIT_SUCCESS:
            return ret
        
        #
        # FLASH SUCCESS
        #

        #
        # Wait for chip to reset
        #
        ###time.sleep(20) # really? so long? TODO: Revisit
        time.sleep(10)
        
        #
        # Delete the the current recovery file
        #
        os.remove(ATMEL_FW_RECOVERY_FILE)

        #
        # Delete the the enc firmware per our agreement with Atmel
        #
        self.deleteFirmware(fw_path)
        
        print '[AtmelTouchFwFlash] - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
        TLOG.logger.info('AtmelTouchFwFlash::run() -----------------END-----------------\n')

        return EXIT_SUCCESS
        

#==========================================================================================================
# AtmelTouchFwFlashTask - Class for Loading Atmel Touch XCFG Update
#==========================================================================================================        
class AtmelTouchXcfgLoadTask:
    up_path=''
    xcfg_name=''
    controller=''
    serial=''
    hwid=''
    
    def __init__(self,path,name,controller,serial,hwid):
        print 'AtmelTouchXcfgLoadTask Constructed'
        TLOG.logger.info('AtmelTouchXcfgLoadTask Constructed')
        # This is the manifest's version of the up path - has quotes
        self.up_path = path.strip('\'')
        # This is the manifest's version of the up name - has quotes
        self.fw_name_dot_up = name.strip('\'')
        self.controller = controller
        self.serial = serial
        self.hwid = hwid

    def run(self):
        
        TLOG.logger.info('AtmelTouchXcfgLoadTask::run() -----------------START---------------')

        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        
        if ADBG: print 'AtmelTouchXcfgLoadTask::run() ', self.up_path, self.xcfg_name, self.controller
        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        
        if ADBG: print '==============================================================='        

        try:
            if PUDBG: pudb.set_trace()
        except:
            pass
        
        usb_devid_tmp = 'usb:'  + self.controller

        up_path = self.up_path
        up_fw_path =  os.path.join(self.up_path,FWDIR_NAME)
        
         
        tskcfg = Configs.UpdateTaskConfig(up_path)
        #
        # Manifest's .XCFG configuration file
        #
        xcfg_name = tskcfg.getAtmelCfgName()
        xcfg_name = xcfg_name.strip('\'')
        xcfg_path = (os.path.join(up_fw_path ,xcfg_name))

        if ADBG: print 'usb_devid=',usb_devid_tmp
        if ADBG: print 'up_path=',up_path
        if ADBG: print 'up_fw_path=',up_fw_path
        if ADBG: print 'xcfg_name=',xcfg_name
        if ADBG: print 'xcfg_path=',xcfg_path

        #
        # Check for mxt-app always .. absence is an unrecoverable error
        #
        if not utils.doesFileExist(MXT_APP):
            print '[AtmelTouchXcfgLoadTask] - Bad OP - No MXT_APP found!'
            TLOG.logger.error('AtmelTouchXcfgLoadTask::run() - Bad OP - No MXT_APP found!')
            raise RuntimeError('Bad OP - No MXT_APP utility found!')

        
        ##################
        # PREPARE XCFG #
        ##################

        print 'Looking for XCFG file :', xcfg_path
        TLOG.logger.info('AtmelTouchXcfgLoadTask::run() Looking for XCFG file : '+xcfg_path)

        if not utils.doesFileExist(xcfg_path):
            print '[AtmelTouchXcfgLoadTask] - Bad UP - No XCFG file found!'
            TLOG.logger.error('AtmelTouchXcfgLoadTask::run() - Bad UP - No XCFG file found!')
            raise RuntimeWarning('Bad UP - No XCFG file found!')
        
        print 'Found XCFG file : ', xcfg_name
        TLOG.logger.info('AtmelTouchXcfgLoadTask::run() Found XCFG file : ' + xcfg_name)
        #


        ##########################
        # RECOVERY FILE CREATE #
        ##########################
        
        print 'Creating recovery file : ', ATMEL_XCFG_RECOVERY_FILE
        TLOG.logger.info('AtmelTouchXcfgLoadTask::run() - Creating recovery file : ' + ATMEL_XCFG_RECOVERY_FILE)

        try:
            up_path_line = UP_PATH + "=" + up_path + "\n"
            xcfg_path_line = XCFG_PATH + "=" + xcfg_path + "\n"
            chip_serial_line = CHIP_SERIAL + "=" + self.serial + "\n"
            #
            f = open(ATMEL_XCFG_RECOVERY_FILE,'w')
            f.write(up_path_line)
            f.write(xcfg_path_line)
            f.write(chip_serial_line)
            f.close()
            
        except IOError:
            print 'Creating recovery file failed'
            TLOG.logger.info('AtmelTouchXcfgLoadTask::run() - Creating recovery file failed')
            # Try remove to be certain
            os.remove(ATMEL_XCFG_RECOVERY_FILE)
            raise RuntimeWarning('Creating recovery file failed')
        #         

        
        ######################
        # SANITY CHECK CHIP #
        ######################      
        #
        # We had an Atmel chip earlier when we assigned tasks - check we still have it
        #
        tmp_controller = self.controller
        self.controller = utils.getUsbDevidForSerial(self.serial)
                    
        if (self.controller=='UNKNOWN'):
            print 'Chip not present any more - Devid: ', tmp_controller,' Serial :',self.serial
            TLOG.logger.warn('AtmelTouchXcfgLoadTask::run() - Chip not present any more - '\
                             +' Old Devid: '+tmp_controller\
                             +' Serial: '+self.serial)
            if CHIP_SANITY_MEANS_REBOOT:
                # Exit and reboot
                return utils.ERROR_CODE_ATMEL_DEVICE_NOT_EXIST
            else:
                # Remove the recovery file and abandon task
                os.remove(ATMEL_XCFG_RECOVERY_FILE)
                raise RuntimeWarning('Chip not present any more')
            #
        else:
            print 'Chip still present - Old Devid: ', tmp_controller,' Devid: ',self.controller,' Serial: ',self.serial
            TLOG.logger.info('AtmelTouchXcfgLoadTask::run() - Chip still present - '\
                             +' Old Devid: '+tmp_controller\
                             +' Devid: '+self.controller\
                             +' Serial: '+self.serial)
            
        #
        # Set up usb devid in mxt-app format: 'usb:hhh-ddd'
        #
        usb_devid_cfg = 'usb:'  + self.controller
        #

                
        ##############
        # XCFG Update #
        ##############
        TLOG.logger.info('AtmelTouchFwFlash::run() -----------------ENUMERATION------------\n')
        utils.find614X()
        # Need delay, if not the mxt-app will encounter errno 19 sometimes
        time.sleep(20)
	
        #Configuration Loading
        print "[AtmelTouchFwFlash] - Loading Configuration to device: " + usb_devid_cfg
        mxt_app_load_cmd = MXT_APP +  ' -d '+ usb_devid_cfg + ' --load ' + xcfg_path                                        

        ret = utils.runSubprocess(mxt_app_load_cmd,'MXT-APP')
                 
        if ret != EXIT_SUCCESS:
            #
            # Bomb out and do a recovery
            #
            return ret
                
        time.sleep(10)
        
        #
        # Delete the the current recovery file
        #
        os.remove(ATMEL_XCFG_RECOVERY_FILE)

        print '[AtmelTouchFwFlash] - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
        TLOG.logger.info('AtmelTouchFwFlash::run() -----------------END-----------------\n')

        return EXIT_SUCCESS
    

       
##==========================================================================================================
#AtmelTouchFwFlashRecovery - Class for the Atmel Recovery Task for Touch Firmware Flash
##==========================================================================================================        
class AtmelTouchFwFlashRecoveryTask:
    
    def __init__(self):
        TLOG.logger.info('AtmelTouchFwFlashRecovery Constructed')
          
    def run(self,fw_path):
        TLOG.logger.info('AtmelTouchFwFlashRecovery::run()-----------------START---------------')
        fw_file_path = fw_path.rstrip('\'')
        
        #ret = utils.doAtmelRecover_old(fw_path,conf)
        mon_list = utils.find614Xin211Dmode()
        #flash firmware
        if (len(mon_list) != 0 ):
                for mon in mon_list:
        	    # Need delay, if not the mxt-app will encounter errno 19 sometimes
                    time.sleep(20)            
                    Atmel_Recover_cmd = MXT_APP + ' -d ' + str(mon)  + ' --flash ' + fw_file_path + ' --verbose=2'            
                    print "[doAtmelRecover] - This will take some time,Please wait ..."
                    
                    ret = utils.runSubprocess(Atmel_Recover_cmd,'ATMEL_RECOVER')
                    if ret != 0:
                        return ret
                        
        else:
                    Atmel_Recover_cmd = MXT_APP + ' --flash ' + fw_file_path
                    ret = utils.runSubprocess(Atmel_Recover_cmd,'ATMEL_RECOVER')
        
        print 'AtmelTouchFwFlashRecovery Done.' + '{' + str(ret) + '}'
        
        TLOG.logger.info('AtmelTouchFwFlashRecovery::run() - Deleting Atmel Firmware Recovery file.' )
        utils.runShell('rm ' + ATMEL_FW_RECOVERY_FILE)
        
        TLOG.logger.info('AtmelTouchFwFlashRecovery::run()-----------------END-----------------\n')
        print 'System is rebooting now...'
        utils.runShell('reboot')
        
        #return EXIT_SUCCESS
        
##==========================================================================================================
#AtmelTouchXcfgLoadRecovery - Class for the Atmel Recovery Task for Touch XCFG Load
##==========================================================================================================        
class AtmelTouchXcfgLoadRecoveryTask:

    def __init__(self):
        TLOG.logger.info('AtmelTouchXcfgLoadRecovery Constructed')
        
    def run(self,conf):
        TLOG.logger.info('AtmelTouchXcfgLoadRecovery::run()-----------------START---------------')
        xcfg_path = conf.rstrip('\'')
        
        #ret = utils.doAtmelRecover_old(fw_path,conf)
        mon_list = utils.find614X()
        #Load config
        if (len(mon_list) != 0 ):
                for mon in mon_list:
		    # Need delay, if not the mxt-app will encounter errno 19 sometimes
		    time.sleep(20)
                    print "[doAtmelRecover] - Loading Configuration now..."
                    mxt_app_load_cmd = MXT_APP +  ' -d'+ str(mon) + ' --load ' + xcfg_path                                            
                    #mxt_app_load_cmd = MXT_APP + ' --load ' + cfg_path 
                    ret = utils.runSubprocess(mxt_app_load_cmd,'MXT-APP')
                    # Ah but then?
                    mxt_app_bkup_cmd = MXT_APP + ' -W -T6 -r1 55 '
                    ret = utils.runSubprocess(mxt_app_bkup_cmd,'MXT-APP')
                    if ret != 0:
                        return ret
                    
        print 'AtmelTouchXcfgLoadRecovery Done.' + '{' + str(ret) + '}'
        
        TLOG.logger.info('AtmelTouchXcfgLoadRecovery::run() - Deleting Atmel Recovery file.' )
        utils.runShell('rm ' + ATMEL_XCFG_RECOVERY_FILE)
        
        TLOG.logger.info('AtmelTouchXcfgLoadRecovery::run()-----------------END-----------------\n')
        print 'System is rebooting now...'
        utils.runShell('reboot')
        
        #return EXIT_SUCCESS


