#!/usr/bin/env python
'''
Module Name:    op_install.py

Description:    This is a helper script during MUS op installations
Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2015 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:    

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 25 September 2013

Feb 7 2015
- Version 2.1
'''

import os,subprocess
import platform
import logging
import datetime

#
# This script is really the only Architecture dependent part of the install. One of the reasons we will keep
# the RPM as .noarch
#
# Also the scripts to TAR up the binaries will now have binaries with _x64 suffixes in some cases so as to hold x64
# versions of the binaries - where available.
#
def on_x86_64bit():
    '''
    32-BIT
    =====
    ('SuSE Linux Enterprise Desktop ', '11', 'i586')
    ('SuSE', '11', 'i586')
    
    64-BIT
    =====
    ('SuSE Linux Enterprise Desktop ', '12', 'x86_64')
    ('SuSE', '12', 'x86_64')
    '''
    linux_dist = platform.linux_distribution(full_distribution_name=0)
    print 'Linux Distribution:', linux_dist
    if linux_dist[2]=='x86_64':
        return True
    return False

#===============================================================================
# #method to run any shell commands from python
#===============================================================================
def runCmd(cmd):
    logger.info(cmd)   
    process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE)
    while True:
        line = process.stdout.readline()
        if not line:
            break
        line = line.strip("\n")
        logger.info(line)
        #print line

#===============================================================================
# #method to get current build number
#===============================================================================
def getBuildNumber() :
    process = subprocess.Popen('cat /tmp/.build_number',shell=True, stdout=subprocess.PIPE)
    while True:
        line = process.stdout.readline()
        if not line:
            break
        line = line.strip("\n")
        return line

'''
Linux-3.0.76-0.9-pae-i686-with-SuSE-11-i586
or
Linux-3.12.36-38-default-x86_64-with-SuSE-12-x86_64
'''
PLAT = platform.platform()
VER = getBuildNumber()
if VER is None: VER = '2.1'
OP_LOG_FILE_DIR = '/var/log/toshiba/mus/op_installation'
if not os.path.exists(OP_LOG_FILE_DIR): OP_LOG_FILE_DIR = '/tmp/'
LOG_NAME = 'mus_op_'+ str(VER)
UP_RPMS_DIR = '/opt/toshiba/mus/RpmUpdates'
ON_64BIT = on_x86_64bit()
   
#Create the logger object
logger = logging.getLogger(LOG_NAME)
logger.setLevel(logging.INFO)  

# make some handlers
date_now = datetime.datetime.now()
logname = LOG_NAME + '-' + str(date_now) + '.log'

name = os.path.join(OP_LOG_FILE_DIR,logname)

#print 'Logging to file '+ name
console_handler = logging.StreamHandler() # by default, sys.stderr
file_handler = logging.FileHandler(name)

# set logging levels
console_handler.setLevel(logging.WARNING)
file_handler.setLevel(logging.INFO)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s:[%(levelname)s] - %(message)s')

# add formatter to file_handler
file_handler.setFormatter(formatter)
console_handler.setFormatter(formatter)
  
# add handlers to logger
logger.addHandler(console_handler)
logger.addHandler(file_handler)      


#===============================================================================
# #Basically , just create the file system at the start and then
# #Register the Monitor Upgrade Services
#===============================================================================

print '\nStarting Monitor Upgrade Service installation...\n'
logger.info('Starting installation...')
print 'Platform:', PLAT
logger.info('Platform: ' + PLAT)

if ON_64BIT:
    print 'This is the Architecture dependent installer for MUS 32-bit - But this machine is 64-bit - Exiting'
    logger.error('This is the Architecture dependent installer for MUS 32-bit - But this machine is 64-bit - Exiting')


#
# Arch dependent
#


#####################
# ELO/SAMPO Utilities #
#####################
#
# Not expected to have 64-bit analogs but also Not used on 64-bit SLE 12 TCXD version
#
print ('Extracting  ELO TOOLS...') 
logger.info('Extracting  ELO TOOLS...')          
print 'Extracting /opt/toshiba/MonitorUpgrade/op/4820/ELO/elo.tar...'
runCmd ('tar -C /opt/toshiba/mus/op_utils/elo/ -xvf  /opt/toshiba/mus/op_utils/elo/elo.tar')
print 'moving elo binaries to /usr/bin/'
runCmd ('mv  /opt/toshiba/mus/op_utils/elo/elo_download /opt/toshiba/mus/op_utils/elo/elo_config /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/elo/FwTouchUpgradeUtil /usr/bin/')
runCmd('chmod +x /usr/bin/elo_download /usr/bin/elo_config /usr/bin/FwTouchUpgradeUtil')
#runCmd ('rm -rf  /opt/toshiba/mus/op_utils/elo/elo.tar')
#
# Not expected to have 64-bit analogs but Not used on 64-bit SLE 12 MUS version
#
print ('Extracting  SAMPO TOOLS...') 
logger.info('Extracting  SAMPO TOOLS...')
runCmd ('tar -C /opt/toshiba/mus/op_utils/sampo -xvf /opt/toshiba/mus/op_utils/sampo/sampo.tar')
#runCmd ('mv  /opt/toshiba/mus/op_utils/sampo/4820MonUtil /usr/bin/')
#runCmd ('rm -rf /opt/toshiba/mus/op_utils/sampo/sampo.tar')

###############
# Helper Utilities #
###############

#
# Should have 64-bit analogs for 64-bit SLE 12 version
#
print ('Extracting  MUS HLPRS TOOLS...') 
logger.info('Extracting  MUS_HLPRS TOOLS...')
#extract
runCmd ('tar -C /opt/toshiba/mus/op_utils/hlprs -xvf /opt/toshiba/mus/op_utils/hlprs/mus_hlprs.tar')
# 32-bit binaries
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/rstelo2  /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/lstmon2  /usr/bin/')
#scripts
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/chk_elo_patch  /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/chk_fousb  /usr/bin/')
runCmd ('chmod 755  /usr/bin/chk_elo_patch')
runCmd ('chmod 755 /usr/bin/chk_fousb')
#
print ('Extracting  AIPFWUPDATE TOOL...')
logger.info('Extracting  AIPFWUPDATE TOOL...')
#extract
runCmd ('tar -C /opt/toshiba/mus/op_utils/aipfwupdate -xvf /opt/toshiba/mus/op_utils/aipfwupdate/aipfwupdate.tar')

# 32-bit binary
runCmd ('mv  /opt/toshiba/mus/op_utils/aipfwupdate/aipfwupdate  /usr/bin/')
    
print ('Extracting  MXTAPP TOOL...') 
logger.info('Extracting  MXTAPP TOOL...')
#extract
runCmd ('tar -C /opt/toshiba/mus/op_utils/mxtapp -xvf /opt/toshiba/mus/op_utils/mxtapp/mxtapp.tar')

runCmd ('mv  /opt/toshiba/mus/op_utils/mxtapp/mxt-app  /usr/bin/')


#
# Arch independent from here down
#


###########
# Licenses #
###########

print ('Extracting  License files...')
logger.info('Extracting License files...')
runCmd ('tar -C /opt/toshiba/mus/LA/ -xvf /opt/toshiba/mus/LA/LA.tar')

runCmd ('rm -rf /opt/toshiba/mus/LA/LA.tar')

#===============================================================================
# Creating magic directory for UP RPMS
# This is where we drop all new UP RPMS
#===============================================================================
print 'Creating MUS directory...'
logger.info('Creating MUS directory...')
logger.info('Checking directory -/opt/toshiba/mus')

if os.path.exists('/opt/toshiba/mus/'):
        #print '/opt/toshiba/MonitorUpgrade/mus/scripts/  exist!'
        logger.info('/opt/toshiba/mus/ exist!')
        print ('Creating UP RPMS directory...')
        logger.info('Creating UP RPMS directory')
        
        if  os.path.exists(UP_RPMS_DIR) != True :
            runCmd ('mkdir '  + UP_RPMS_DIR)
        

#===============================================================================
# Creating UnInstall script
#===============================================================================
#if os.path.exists('/opt/toshiba/mus/'):
#        
#        #print '/opt/toshiba/MonitorUpgrade/mus/scripts/  exist!'
#        logger.info('Creating UnInstall script')
#       
#        print ('Creating UnInstall script') 
#                
#        runCmd ('tar -xvf opt/toshiba/mus/mus_op_unistall.tar' )
#    
#===============================================================================
# #Create a copy of our service to /etc/init.d`
# #This will enable our script to be invoke during boot up of the machine
# #Note: MUST make sure the copy in the /etc/init.d should not have the extension 
# ".py" or else this won't work
#===============================================================================
#print 'Adding init script to /etc/init.d/...'
logger.info('Adding init script to /etc/init.d/...')

#print 'Copying MonitorUpgradeSvc...'
runCmd('cp -p /usr/local/lib/python2.6/site-packages/initscript/MonitorUpgradeSvc.py /etc/init.d/MonitorUpgradeSvc')


#===============================================================================
# #Now Register the service using chkconfig
# #Opted to use this method instead of insserv,
# #They pretty much do the same function
#
# WILL THIS ALL WORK ON SLE 12 WITH SYSTEMD ???
#
#===============================================================================

runCmd('chmod +x /etc/init.d/MonitorUpgradeSvc')
#runCmd('chown root:root /etc/init.d/MonitorUpgradeSvc')

print 'Registering MonitorUpgradeSvc...'
logger.info('Registering MonitorUpgradeSvc...')

runCmd('chkconfig -a MonitorUpgradeSvc')
runCmd('chkconfig MonitorUpgradeSvc on')


#===============================================================================
# #DONE!
#===============================================================================
logger.info('Installation\Upgrade Complete!')

